﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MaxIm;

namespace MxPlugin_AddConstant
{

    // COM Required Attributes for a custom coclass Interface
    // You must replace this GUID with a custom value.
    // Go to "Tools" > "Create GUID" to create your own.
    [Guid("FF4910B6-8891-476D-B59E-6EECEAA21A59")]
    [CoClass(typeof(PlugIn))]
    [ComImport]
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
    public interface _PlugIn
    {
        [DispId(0)] void DoModal([In, MarshalAs(UnmanagedType.Interface)] MaxIm.Document doc);

        [DispId(1)]
        [return: MarshalAs(UnmanagedType.BStr)]
        string Name();
    }

    // The coclass implementation of the COM Interface.
    // You must replace this GUID with a custom value.
    // Go to "Tools" > "Create GUID" to create your own.
    [Guid("72DAF4E7-3C0E-4A65-9E03-7AD2AE11E058")]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    public class PlugIn : _PlugIn
    {
        /*!
         * \brief Class constructor
         * Default class contructor must be implemented for COM objects.
         */
        public PlugIn()
        {
            // Initialize the COM object.
        }

        /*!
         * \brief Function executed on PlugIn execute call.
         * DoModal is the function call passed through the COM to the PlugIn. Takes the currently open MaxIm DL document as an argument. 
         * \param MaxIm.Document doc Document to execute the plugin on. Provides direct access to the Image Buffer SAFEARRAY, rather than through a VARIANT array. 
         *        Is the open document with current focus in MaxIm DL.
         */
        public void DoModal(MaxIm.Document doc)
        {
            // Do stuff here.

            int W = doc.XSize;
            int H = doc.YSize;

            
            bool isColor = doc.Color;

            if (isColor)
            {
                float[,,] b = (float[,,])(object)(doc.ImageArray);

                for (int I = 0; I < W; I++)
                {
                    for (int J = 0; J < H; J++)
                    {
                        b[0, I, J] = b[0, I, J] + 50.0F;
                        b[1, I, J] = b[1, I, J] + 150.0F;
                        b[2, I, J] = b[2, I, J] + 200.0F;
                    }
                }
                doc.ImageArray = b;
            }
            else
            {
                float[,] b = (float[,])(object)(doc.ImageArray);

                for (int I = 0; I < W; I++)
                {
                    for (int J = 0; J < H; J++)
                    {
                        b[I, J] = b[I, J] + 100.0F;
                    }
                }
                doc.ImageArray = b;
            }

            doc.Modified = true;

        }

        /*!
         * \brief Returns the name of the PlugIn for use in the MaxIm DL GUI.
         * Returns the name of the PlugIn for use in the MaxIm DL GUI.
         */
        public string Name()
        {
            return "MaxIm DL Sample C# Plugin - AddConstant";
        }
    }
}
